/*
 * Chax.m
 *
 * Copyright (c) 2007-2009 Kent Sutherland
 * 
 * Permission is hereby granted, free of charge, to any person obtaining a copy of
 * this software and associated documentation files (the "Software"), to deal in
 * the Software without restriction, including without limitation the rights to use,
 * copy, modify, merge, publish, distribute, sublicense, and/or sell copies of the
 * Software, and to permit persons to whom the Software is furnished to do so,
 * subject to the following conditions:
 * 
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 * 
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS
 * FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR
 * COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER
 * IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
 * CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 */

#import <IOKit/pwr_mgt/IOPMLib.h>
#import <IOKit/pwr_mgt/IOPM.h>
#import "Chax.h"
#import "StatusChangeController.h"
#import "LogViewerController.h"
#import "ActivityWindowController.h"
#import "UpdateController.h"
#import "Chax_AIMLGenerator.h"
#import "Chax_JabberXHTMLGenerator.h"
#import "Chax_SecureWindow.h"
#import "UnifiedPeopleListController.h"
#import "ScreenSaver.h"
#import "DonateWindowController.h"

NSString *ChaxBundleIdentifier = @"com.ksuther.chax";

static NSInteger kChaxDonateRequestFirstInterval = 604800;
static NSInteger kChaxDonateRequestSecondInterval = 2678400;

static NSArray *_chaxMenuItems = nil;
static NSString *_previousMessage = nil;
static BOOL _screensaverAwayed = NO;

@implementation Chax

+ (void)load
{
	[Chax_AIMLGenerator poseAsClass:[AIMLGenerator class]];
	[Chax_JabberXHTMLGenerator poseAsClass:[JabberXHTMLGenerator class]];
	[Chax_SecureWindow poseAsClass:[SecureWindow class]];
	
	[StatusChangeController sharedController]; //Load the Growl framework and register for Presentity status changes
	
	[Prefs setKnockKnock:![Chax boolForKey:@"SkipNewMessageNotification"]];
	
	[self registerDefaults];
	[self addMenuItems];
	
	[[UpdateController sharedController] setAutomaticCheckingEnabled:[Chax boolForKey:@"AutoUpdateChax"]];
	
	[[NSDistributedNotificationCenter defaultCenter] addObserver:self selector:@selector(screensaverNotificationReceived:) name:@"com.apple.screensaver.didstart" object:nil];
	[[NSDistributedNotificationCenter defaultCenter] addObserver:self selector:@selector(screensaverNotificationReceived:) name:@"com.apple.screensaver.didstop" object:nil];
	
	[[NSNotificationCenter defaultCenter] addObserver:self selector:@selector(notificationReceived:) name:NSApplicationDidFinishLaunchingNotification object:nil];
	
	//Display the unified contact list if it has never been shown before
	if ([[NSUserDefaults standardUserDefaults] objectForKey:@"Chax.Visible"] == nil || [[UnifiedPeopleListController sharedController] prefVisible]) {
		//Calling showWindow: instead of displayIfPrefVisible ensures that the window is made key if no other windows are opened
		[[UnifiedPeopleListController sharedController] performSelector:@selector(showWindow:) withObject:nil afterDelay:0.0];
	}
}

+ (void)registerDefaults
{
	//Ask whether or not update checking should be enabled
	if ([Chax objectForKey:@"AutoUpdateChax"] == nil || [Chax objectForKey:@"AutoUpdateIncludeVersionInfo"] == nil) {
		[[UpdateController sharedController] performSelector:@selector(promptForAutomaticUpdates) withObject:nil afterDelay:4.0];
	}
	
	//Default menu badge showing to true
	if ([Chax objectForKey:@"ShowMenuBadge"] == nil) {
		[Chax setBool:YES forKey:@"ShowMenuBadge"];
	}
}

+ (void)addMenuItems
{
	NSImage *badgeImage = [[[NSImage alloc] initByReferencingFile:[[NSBundle bundleWithIdentifier:ChaxBundleIdentifier] pathForImageResource:@"menu-badge"]] autorelease];
	NSMutableArray *menuItems = [NSMutableArray array];
	NSMenuItem *menuItem;
	NSMenu *viewMenu = [[[NSApp mainMenu] itemAtIndex:3] submenu];
	NSMenu *buddiesMenu = [[[NSApp mainMenu] itemAtIndex:4] submenu];
	NSMenu *audioMenu = [[[NSApp mainMenu] itemAtIndex:5] submenu];
	
	//Toggle text status menu item
	menuItem = [viewMenu insertItemWithTitle:ChaxLocalizedString(@"show_text_status") action:@selector(chax_toggleTextStatus:) keyEquivalent:@"" atIndex:5];
	[menuItem setTag:SHOW_TEXT_STATUS_MENU_ITEM];
	[menuItem setState:NSOnState];
	[menuItems addObject:menuItem];
	
	//Send camera snapshot menu item
	menuItem = [buddiesMenu insertItemWithTitle:ChaxLocalizedString(@"send_camera_snapshot") action:@selector(chax_sendCameraSnapshot:) keyEquivalent:@"j" atIndex:12];
	[menuItem setTag:SEND_CAMERA_SNAPSHOT_MENU_ITEM];
	[menuItem setKeyEquivalentModifierMask:NSCommandKeyMask | NSAlternateKeyMask];
	[menuItems addObject:menuItem];
	
	//Always on top menu item
	menuItem = [[NSApp windowsMenu] insertItemWithTitle:ChaxLocalizedString(@"always_on_top") action:@selector(chax_toggleAlwaysOnTop:) keyEquivalent:@"" atIndex:3];
	[menuItem setTag:ALWAYS_ON_TOP_MENU_ITEM];
	[menuItems addObject:menuItem];
	
	//Log viewer menu item
	menuItem = [[NSApp windowsMenu] addItemWithTitle:ChaxLocalizedString(@"log_viewer") action:@selector(showWindow:) keyEquivalent:@""];
	[menuItem setTag:LOG_VIEWER_MENU_ITEM];
	[menuItem setTarget:[LogViewerController sharedController]];
	[menuItems addObject:menuItem];
	
	//Activity window menu item
	menuItem = [[NSApp windowsMenu] addItemWithTitle:ChaxLocalizedString(@"activity") action:@selector(showWindow:) keyEquivalent:@"a"];
	[menuItem setTag:ACTIVITY_WINDOW_MENU_ITEM];
	[menuItem setTarget:[ActivityWindowController sharedController]];
	[menuItem setKeyEquivalentModifierMask:NSCommandKeyMask | NSAlternateKeyMask];
	[menuItems addObject:menuItem];
	
	//All contacts menu item
	menuItem = [[NSApp windowsMenu] addItemWithTitle:ChaxLocalizedString(@"all_contacts") action:@selector(showPeopleListController:) keyEquivalent:@"1"];
	[menuItem setTag:ALL_CONTACTS_MENU_ITEM];
	[menuItem setTarget:[Fezz sharedInstance]];
	[menuItem setRepresentedObject:[[UnifiedPeopleListController sharedController] representedAccount]];
	[menuItem setKeyEquivalentModifierMask:NSCommandKeyMask | NSAlternateKeyMask];
	[menuItems addObject:menuItem];
	
	//Mute alert sounds menu item
	menuItem = [audioMenu addItemWithTitle:ChaxLocalizedString(@"mute_alert_sounds") action:@selector(chax_toggleMuteAlerts:) keyEquivalent:@""];
	[menuItem setTag:MUTE_ALERTS_MENU_ITEM];
	[menuItem setTarget:self];
	[menuItems addObject:menuItem];
	
	//By handle menu item
	menuItem = [[[viewMenu itemAtIndex:13] submenu] insertItemWithTitle:ChaxLocalizedString(@"by_handle") action:@selector(chax_sortByHandle:) keyEquivalent:@"" atIndex:3];
	[menuItem setTag:BY_HANDLE_MENU_ITEM];
	[menuItems addObject:menuItem];
	
	if ([Chax boolForKey:@"ShowMenuBadge"]) {
		[menuItems makeObjectsPerformSelector:@selector(setImage:) withObject:badgeImage];
	}
	
	_chaxMenuItems = [[NSArray alloc] initWithArray:menuItems];
}

+ (NSArray *)menuItems
{
	return _chaxMenuItems;
}

+ (void)screensaverNotificationReceived:(NSNotification *)note
{
	if ([Chax boolForKey:@"AwayOnScreenSaver"] && [[note name] isEqualToString:@"com.apple.screensaver.didstart"]) {
		if ([Service myStatus] == 4 || [Service myStatus] == 2) {
			_previousMessage = [[Service myStatusMessage] retain];
			[Service setStatus:3 message:[Chax stringForKey:@"AwayOnScreenSaverMessage"]];
			_screensaverAwayed = YES;
		}
	} else if (_screensaverAwayed && [Chax boolForKey:@"UnawayOnScreenSaver"] && [[note name] isEqualToString:@"com.apple.screensaver.didstop"]) {
		if ([Service myStatus] == 3) {
			[Service setStatus:4 message:_previousMessage];
			[_previousMessage release];
		}
		_screensaverAwayed = NO;
	}
}

+ (void)notificationReceived:(NSNotification *)note
{
	if ([[note name] isEqualToString:NSApplicationDidFinishLaunchingNotification]) {
		//Check if the donation window should be shown
		NSString *msg = nil;
		NSInteger time = [Chax integerForKey:@"NextDonateRequest"];
		NSInteger count = [Chax integerForKey:@"DonateCount"];
		
		if (count == 0) {
			[Chax setInteger:[NSDate timeIntervalSinceReferenceDate] + kChaxDonateRequestFirstInterval forKey:@"NextDonateRequest"];
			[Chax setInteger:1 forKey:@"DonateCount"];
		} else if (count == 1 && [NSDate timeIntervalSinceReferenceDate] > time) {
			msg = @"donate_msg_1";
			[Chax setInteger:2 forKey:@"DonateCount"];
			[Chax setInteger:[NSDate timeIntervalSinceReferenceDate] + kChaxDonateRequestSecondInterval forKey:@"NextDonateRequest"];
		} else if (count == 2 && [NSDate timeIntervalSinceReferenceDate] > time) {
			msg = @"donate_msg_2";
			[Chax setInteger:-1 forKey:@"DonateCount"];
			[Chax setInteger:0 forKey:@"NextDonateRequest"];
		}
		
		if (msg) {
			DonateWindowController *controller = [[DonateWindowController alloc] initWithMessage:ChaxLocalizedString(msg)];
			[controller showWindow:nil];
		}
	}
}

+ (void)undimScreenAndStopScreenSaver
{
	//Post a mouse moved event
	CGPoint mouseLocation;
	HIGetMousePosition(kHICoordSpaceScreenPixel, NULL, &mouseLocation);
	CGEventRef event = CGEventCreateMouseEvent(NULL, kCGEventMouseMoved, mouseLocation, kCGMouseButtonLeft);
	CGEventPost(kCGHIDEventTap, event);
	CFRelease(event);
	
	static io_connect_t mgt = 0;
	static mach_port_t masterPort = 0;
	
	if (masterPort == 0) {
		if (IOMasterPort(MACH_PORT_NULL, &masterPort) != kIOReturnSuccess) {
			return;
		}
		
		if (!(mgt = IOPMFindPowerManagement(masterPort))) {
			return;
		}
	}
	
	//Reset dim time
	unsigned long savedDimTime;
	
	if (IOPMGetAggressiveness(mgt, kPMMinutesToDim, &savedDimTime) != kIOReturnSuccess) {
		return;
	}
	
	if (IOPMSetAggressiveness(mgt, kPMMinutesToDim, 0) != kIOReturnSuccess) {
		return;
	}
	
	if (IOPMSetAggressiveness(mgt, kPMMinutesToDim, savedDimTime) != kIOReturnSuccess) {
		return;
	}
	
	//Stop screensaver
	ProcessSerialNumber number;
	number.highLongOfPSN = kNoProcess;
	number.lowLongOfPSN = 0;
	
	while ( (GetNextProcess(&number) == noErr) ) {
		CFStringRef name;
		if ((CopyProcessName(&number, &name) == noErr) && [(NSString *)name isEqualToString:@"ScreenSaverEngine"] && [[ScreenSaverController controller] screenSaverIsRunning]) {
			[[ScreenSaverController controller] screenSaverStopNow];
		}
		[(NSString *)name release];
	}
}

+ (void)chax_toggleMuteAlerts:(id)sender
{
	[Chax setBool:([(NSMenuItem *)sender state] != NSOnState) forKey:@"MuteAlerts"];
}

+ (BOOL)validateMenuItem:(NSMenuItem *)menuItem
{
	if ([menuItem tag] == MUTE_ALERTS_MENU_ITEM) {
		[menuItem setState:[Chax boolForKey:@"MuteAlerts"]];
	}
	return YES;
}

#pragma mark -
#pragma mark Chax Defaults Reading/Writing
#pragma mark -


+ (BOOL)boolForKey:(NSString *)key
{
	return CFPreferencesGetAppBooleanValue((CFStringRef)key, (CFStringRef)ChaxBundleIdentifier, nil);
}

+ (int)integerForKey:(NSString *)key
{
	return CFPreferencesGetAppIntegerValue((CFStringRef)key, (CFStringRef)ChaxBundleIdentifier, nil);
}

+ (NSData *)dataForKey:(NSString *)key
{
	id object = (id)CFPreferencesCopyAppValue((CFStringRef)key, (CFStringRef)ChaxBundleIdentifier);
	if (object && CFGetTypeID(object) == CFDataGetTypeID()) {
		return [object autorelease];
	} else {
		[object release];
		return nil;
	}
}

+ (NSString *)stringForKey:(NSString *)key
{
	id object = (id)CFPreferencesCopyAppValue((CFStringRef)key, (CFStringRef)ChaxBundleIdentifier);
	if (object && CFGetTypeID(object) == CFStringGetTypeID()) {
		return [object autorelease];
	} else {
		[object release];
		return nil;
	}
}

+ (id)objectForKey:(NSString *)key
{
	return [(id)CFPreferencesCopyAppValue((CFStringRef)key, (CFStringRef)ChaxBundleIdentifier) autorelease];
}

+ (void)setBool:(BOOL)value forKey:(NSString *)key
{
	NSUserDefaults *df = [NSUserDefaults standardUserDefaults];
	NSMutableDictionary *dictionary = [[[df persistentDomainForName:ChaxBundleIdentifier] mutableCopy] autorelease];
	
	if (!dictionary) {
		dictionary = [NSMutableDictionary dictionary];
	}
	
	[dictionary setObject:[NSNumber numberWithBool:value] forKey:key];
	[df setPersistentDomain:dictionary forName:ChaxBundleIdentifier];
}

+ (void)setInteger:(int)value forKey:(NSString *)key
{
	NSUserDefaults *df = [NSUserDefaults standardUserDefaults];
	NSMutableDictionary *dictionary = [[[df persistentDomainForName:ChaxBundleIdentifier] mutableCopy] autorelease];
	
	if (!dictionary) {
		dictionary = [NSMutableDictionary dictionary];
	}
	
	[dictionary setObject:[NSNumber numberWithInt:value] forKey:key];
	[df setPersistentDomain:dictionary forName:ChaxBundleIdentifier];
}

+ (void)setObject:(id)value forKey:(NSString *)key
{
	NSUserDefaults *df = [NSUserDefaults standardUserDefaults];
	NSMutableDictionary *dictionary = [[[df persistentDomainForName:ChaxBundleIdentifier] mutableCopy] autorelease];
	
	if (!dictionary) {
		dictionary = [NSMutableDictionary dictionary];
	}
	
	[dictionary setObject:value forKey:key];
	[df setPersistentDomain:dictionary forName:ChaxBundleIdentifier];
}

@end
