/*
 * install_tool.m
 *
 * Copyright (c) 2007-2009 Kent Sutherland
 * 
 * Permission is hereby granted, free of charge, to any person obtaining a copy of
 * this software and associated documentation files (the "Software"), to deal in
 * the Software without restriction, including without limitation the rights to use,
 * copy, modify, merge, publish, distribute, sublicense, and/or sell copies of the
 * Software, and to permit persons to whom the Software is furnished to do so,
 * subject to the following conditions:
 * 
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 * 
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS
 * FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR
 * COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER
 * IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
 * CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 */

#import <Cocoa/Cocoa.h>
#import <Foundation/Foundation.h>
#import <stdio.h>
#import <grp.h>
#import <pwd.h>
#import <sys/stat.h>

int main(int argc, const char *argv[])
{
    NSAutoreleasePool *pool = [[NSAutoreleasePool alloc] init];
	NSString *inputManagersPath = @"/Library/InputManagers";
	NSString *installPath = [inputManagersPath stringByAppendingPathComponent:@"Chax"];
	NSString *userInstallPath = [@"~/Library/InputManagers/Chax" stringByExpandingTildeInPath];
	NSString *simblPluginPath = @"/Library/Application Support/SIMBL/Plugins/Chax.bundle";
	NSString *simblUserPluginPath = [@"~/Library/Application Support/SIMBL/Plugins/Chax.bundle" stringByExpandingTildeInPath];
	NSFileManager *fm = [NSFileManager defaultManager];
	
	if (argc < 2) {
		exit(1);
	}
	
	switch (argv[1][0]) {
		case 'i':
			//Install
			{
				NSString *bundlePath = [[[NSString stringWithUTF8String:argv[0]] stringByDeletingLastPathComponent] stringByAppendingPathComponent:@"Chax"];
				BOOL isDir;
				
				//Get the uid/gid of root:admin
				struct passwd *passwdInfo = getpwnam("root");	
				struct group *groupInfo = getgrnam("admin");
				unsigned int uid = passwdInfo->pw_uid;
				
				if (![fm fileExistsAtPath:inputManagersPath isDirectory:&isDir]) {
					if ([fm createDirectoryAtPath:inputManagersPath attributes:nil]) {
						//Set permissions on the newly-created directory
						const char *path = [inputManagersPath fileSystemRepresentation];
						
						if (chmod(path, S_IRWXU | S_IRGRP | S_IXGRP | S_IROTH | S_IXOTH) != 0) {
							printf("error_permissions");
							return 0;
						}
						
						if (chown(path, uid, groupInfo->gr_gid) != 0) {
							printf("error_permissions");
							return 0;
						}
					} else {
						//Could not create /Library/InputManagers
						printf("error_mkdir");
						return 0;
					}
				} else if (!isDir) {
					// /Library/InputManagers is NOT a directory
					printf("error_notdir");
					return 0;
				}
				
				//Remove SIMBL plugin from /Library/Application Support/SIMBL/Plugins/Chax if it exists
				if ([fm fileExistsAtPath:simblPluginPath isDirectory:&isDir]) {
					if (![fm removeFileAtPath:simblPluginPath handler:nil]) {
						//Error removing Chax SIMBL plugin from /Library/Application Support/SIMBL/Plugins
						printf("error_removing_simbl");
						return 0;
					}
				}
				
				//Remove SIMBL plugin from ~/Library/Application Support/SIMBL/Plugins/Chax if it exists
				if ([fm fileExistsAtPath:simblUserPluginPath isDirectory:&isDir]) {
					if (![fm removeFileAtPath:simblUserPluginPath handler:nil]) {
						//Error removing Chax SIMBL plugin from ~/Library/Application Support/SIMBL/Plugins
						printf("error_removing_simbl");
						return 0;
					}
				}
				
				//Remove Chax from ~/Library/InputManagers if it exists
				if ([fm fileExistsAtPath:userInstallPath isDirectory:&isDir]) {
					if (![fm removeFileAtPath:userInstallPath handler:nil]) {
						//Error removing Chax from ~/Library/InputManagers
						printf("error_removing");
						return 0;
					}
				}
				
				//Remove any existing version of Chax
				if ([fm fileExistsAtPath:installPath isDirectory:&isDir]) {
					if (![fm removeFileAtPath:installPath handler:nil]) {
						//Error removing Chax from /Library/InputManagers
						printf("error_removing");
						return 0;
					}
				}
				
				if (![fm copyPath:bundlePath toPath:installPath handler:nil]) {
					//Error copying Chax to /Library/InputManagers
					printf("error_copying");
				} else {
					//Set the permissions on the copied directory
					NSError *error = nil;
					NSArray *paths = [fm subpathsOfDirectoryAtPath:installPath error:&error];
					
					if (paths == nil || error) {
						printf("error_permissions");
						return 0;
					}
					
					const char *path = [inputManagersPath fileSystemRepresentation];
					
					if (chmod(path, S_IRWXU | S_IRGRP | S_IXGRP | S_IROTH | S_IXOTH) != 0) {
						printf("error_permissions");
						return 0;
					}
					
					if (chown(path, uid, groupInfo->gr_gid) != 0) {
						printf("error_permissions");
						return 0;
					}
					
					path = [installPath fileSystemRepresentation];
					
					if (chmod(path, S_IRWXU | S_IRGRP | S_IXGRP | S_IROTH | S_IXOTH) != 0) {
						printf("error_permissions");
						return 0;
					}
					
					if (chown(path, uid, groupInfo->gr_gid) != 0) {
						printf("error_permissions");
						return 0;
					}
					
					for (NSString *nextPath in paths) {
						path = [[installPath stringByAppendingPathComponent:nextPath] fileSystemRepresentation];
						
						if (chmod(path, S_IRWXU | S_IRGRP | S_IXGRP | S_IROTH | S_IXOTH) != 0) {
							printf("error_permissions");
							return 0;
						}
						
						if (chown(path, uid, groupInfo->gr_gid) != 0) {
							printf("error_permissions");
							return 0;
						}
					}
					
					printf("success_installing");
				}
			}
			break;
		case 'd':
			//Delete
			if (![fm removeFileAtPath:installPath handler:nil]) {
				//Error removing Chax from /Library/InputManagers
				printf("error_removing");
			} else {
				printf("success_removing");
			}
			break;
	}
	
    [pool release];
    return 0;
}