/*
 * LogViewerController.m
 *
 * Copyright (c) 2007-2009 Kent Sutherland
 * 
 * Permission is hereby granted, free of charge, to any person obtaining a copy of
 * this software and associated documentation files (the "Software"), to deal in
 * the Software without restriction, including without limitation the rights to use,
 * copy, modify, merge, publish, distribute, sublicense, and/or sell copies of the
 * Software, and to permit persons to whom the Software is furnished to do so,
 * subject to the following conditions:
 * 
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 * 
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS
 * FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR
 * COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER
 * IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
 * CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 */

#import "LogViewerController.h"
#import "Chax.h"
#import "AGRegex.h"
#import "LogFile.h"

static LogViewerController *_sharedController;

NSString *DeleteLogToolbarItemIdentifier = @"DeleteLogToolbarItemIdentifier";
NSString *ExportLogToolbarItemIdentifier = @"ExportLogToolbarItemIdentifier";
NSString *RevealLogToolbarItemIdentifier = @"RevealLogToolbarItemIdentifier";
NSString *SearchFieldToolbarItemIdentifier = @"SearchFieldToolbarItemIdentifier";

@interface LogViewerController (Private)

- (void)selectPresentity:(Presentity *)presentity;
- (void)selectLog:(NSManagedObject *)log;
- (void)clearSearchField;
- (void)searchTextChanged:(id)sender;

- (void)reloadLogs;
- (void)reloadLogsAndPerformSelector:(SEL)selector withObject:(id)object;
- (void)createLogStructureFromEnumerator:(NSEnumerator *)enumerator;
- (void)resetLogModificationDates;

- (NSManagedObjectContext *)managedObjectContext;

@end

@implementation LogViewerController

+ (LogViewerController *)sharedController
{
	if (!_sharedController) {
		_sharedController = [[LogViewerController alloc] initWithWindowNibName:@"LogViewer"];
	}
	return _sharedController;
}

- (id)initWithWindowNibName:(NSString *)string
{
	if ( (self = [super initWithWindowNibName:string]) ) {
		_firstLoadComplete = NO;
	}
	return self;
}

- (void)dealloc
{
	[[NSNotificationCenter defaultCenter] removeObserver:self];
	[self clearChatView];
	[_toolbar release];
	[_searchField release];
	[super dealloc];
}

- (void)windowDidLoad
{
	[self setWindowFrameAutosaveName:@"LogViewer"];
	
	_toolbar = [[NSToolbar alloc] initWithIdentifier:@"LogToolbar"];
	[_toolbar setDelegate:self];
	[_toolbar setAllowsUserCustomization:YES];
	[_toolbar setAutosavesConfiguration:YES];
	[[self window] setToolbar:_toolbar];
	
	[_personArrayController setSortDescriptors:[NSArray arrayWithObject:[[[NSSortDescriptor alloc] initWithKey:@"name" ascending:YES selector:@selector(caseInsensitiveCompare:)] autorelease]]];
	[_logsArrayController setSortDescriptors:[NSArray arrayWithObject:[[[NSSortDescriptor alloc] initWithKey:@"date" ascending:NO] autorelease]]];
	
	[_chatDOMView initializeWebView];
	[_chatDOMView setSampleChat];
	
	[[NSNotificationCenter defaultCenter] addObserver:self selector:@selector(tableViewNotificationReceived:) name:@"ChaxTableArrowDown" object:nil];
}

- (void)showWindow:(id)sender
{
	BOOL wasVisible = [[self window] isVisible];
	
	[self clearSearchField];
	
	[super showWindow:sender];
	
	if (!wasVisible) {
		[self reloadLogs];
	}
	
	if (_badDate && !_ignoreDates) {
		NSBeginAlertSheet(ChaxLocalizedString(@"fix_log_dates"), ChaxLocalizedString(@"fix_dates"), ChaxLocalizedString(@"update_ignore"), nil, [self window], self, @selector(resetDatesSheetDidEnd:returnCode:contextInfo:), nil, nil, ChaxLocalizedString(@"fix_log_dates_msg"));
	}
}

#pragma mark -
#pragma mark Find
#pragma mark -

- (void)showFindPanel:(id)sender
{
	[[FindPanelController sharedController] showWindow:sender];
	[[self window] makeFirstResponder:_chatDOMView];
}

- (void)findNext:(id)sender
{
	[[FindPanelController sharedController] findNext:sender];
}

- (void)findPrevious:(id)sender
{
	[[FindPanelController sharedController] findPrevious:sender];
}

#pragma mark -
#pragma mark Log Loading
#pragma mark -

- (void)reloadLogs
{
	[self reloadLogsAndPerformSelector:NULL withObject:nil];
}

- (void)reloadLogsAndPerformSelector:(SEL)selector withObject:(id)object
{
	NSInvocation *invocation = nil;
	
	if (selector && object) {
		invocation = [NSInvocation invocationWithMethodSignature:[self methodSignatureForSelector:selector]];
		[invocation setSelector:selector];
		[invocation setTarget:self];
		[invocation setArgument:&object atIndex:2];
		[invocation retainArguments];
	}
	
	[NSThread detachNewThreadSelector:@selector(reloadLogsThread:) toTarget:self withObject:invocation];
}

- (void)reloadLogsThread:(NSInvocation *)invocation
{
	NSAutoreleasePool *pool = [[NSAutoreleasePool alloc] init];
	
	[_loadingProgressIndicator performSelectorOnMainThread:@selector(startAnimation:) withObject:nil waitUntilDone:NO];
	
	[[self managedObjectContext] reset];
	
	[[self window] performSelectorOnMainThread:@selector(display) withObject:nil waitUntilDone:YES];
	
	[self createLogStructureFromEnumerator:[[NSFileManager defaultManager] enumeratorAtPath:[Prefs savedChatPath]]];
	[[self managedObjectContext] processPendingChanges];
	
	if (!_firstLoadComplete) {
		_firstLoadComplete = YES;
	}
	
	[_loadingProgressIndicator performSelectorOnMainThread:@selector(stopAnimation:) withObject:nil waitUntilDone:NO];
	
	if (invocation) {
		[invocation invoke];
	}
	
	[pool release];
}

- (void)createLogStructureFromEnumerator:(NSEnumerator *)enumerator
{
	//Regular expression that searches for a separator and then a date. This is the format of most logs
	AGRegex *primaryRegex = [AGRegex regexWithPattern:@" \\S{1,4} (([0-9]{4}|[0-9]{2}).){3}"];
	
	//Regular expression that searches for just a date
	AGRegex *secondaryRegex = [AGRegex regexWithPattern:@" (([0-9]{4}|[0-9]{2}).){3}"];
	
	//Regular expression that searches for the French date format
	AGRegex *tertiaryRegex = [AGRegex regexWithPattern:@" \\S{2,3} -(([0-9]{4}|[0-9]{2}).){3}"];
	
	NSString *nextFile;
	NSString *savedChatsPath = [Prefs savedChatPath];
	NSMutableDictionary *personEntities = [NSMutableDictionary dictionary];
	
	_badDate = NO;
	
	while ( (nextFile = [enumerator nextObject]) ) {
		if ([[nextFile pathExtension] isEqualToString:@"chat"] || [[nextFile pathExtension] isEqualToString:@"ichat"]) {
			AGRegexMatch *match = [primaryRegex findInString:nextFile];
			NSString *name = nil;
			int location;
			
			//Search for matches on the log formats
			if (!match) {
				//Secondary search
				match = [secondaryRegex findInString:nextFile];
				
				if (!match) {
					//French format
					match = [tertiaryRegex findInString:nextFile];
				}
			}
			
			if (match) {
				//Get the name of the person
				location = [match range].location;
				name = [[nextFile substringToIndex:location] lastPathComponent];
				
				//See if the person is already in the Core Data store
				NSManagedObject *person = [personEntities objectForKey:name];
				
				[[self managedObjectContext] lock];
				
				if (person == nil) {
					//The person doesn't exist, add them
					person = [NSEntityDescription insertNewObjectForEntityForName:@"Person" inManagedObjectContext:[self managedObjectContext]];
					[person setValue:name forKey:@"name"];
					
					[personEntities setObject:person forKey:name];
				}
				
				NSString *path = [savedChatsPath stringByAppendingPathComponent:nextFile];
				NSDictionary *attributes = [[NSFileManager defaultManager] fileAttributesAtPath:path traverseLink:NO];
				NSDate *creationDate = [attributes objectForKey:NSFileCreationDate];
				
				if ([creationDate earlierDate:[NSDate dateWithString:@"2000-01-01 12:00:00 +0000"]] == creationDate) {
					_badDate = YES;
				}
				
				//Add the log file to the Core Data store, naming it with the file creation date
				NSManagedObject *log = [NSEntityDescription insertNewObjectForEntityForName:@"Log" inManagedObjectContext:[self managedObjectContext]];
				[log setValue:[creationDate descriptionWithCalendarFormat:@"%b %d, %Y, %1I:%M %p" timeZone:nil locale:[[NSUserDefaults standardUserDefaults] dictionaryRepresentation]] forKey:@"name"];
				[log setValue:path forKey:@"path"];
				[log setValue:creationDate forKey:@"date"];
				[log setValue:person forKey:@"person"];
				
				[[self managedObjectContext] unlock];
			}
		}
	}
}

- (void)showLogsForPresentity:(Presentity *)presentity
{
	[self clearSearchField];
	[[self window] makeKeyAndOrderFront:nil];
	
	[self reloadLogsAndPerformSelector:@selector(selectPresentity:) withObject:presentity];
}

- (void)selectPresentity:(Presentity *)presentity
{
	NSFetchRequest *fetchRequest = [[[NSFetchRequest alloc] init] autorelease];
	NSEntityDescription *personEntity = [NSEntityDescription entityForName:@"Person" inManagedObjectContext:[self managedObjectContext]];
	
	[fetchRequest setEntity:personEntity];
	[fetchRequest setPredicate:[NSPredicate predicateWithFormat:@"name = %@ || name = %@", [presentity name], [presentity ID]]];
	NSArray *results = [[self managedObjectContext] executeFetchRequest:fetchRequest error:nil];
	
	if ([results count] > 0) {
		[_personArrayController setSelectedObjects:[NSArray arrayWithObject:[results objectAtIndex:0]]];
		[_personTableView scrollRowToVisible:[_personTableView selectedRow]];
		[[self window] display];
	} else {
		NSBeginAlertSheet(ChaxLocalizedString(@"no_logs"), nil, nil, nil, [self window], self, NULL, NULL, NULL, ChaxLocalizedString(@"no_logs_msg"), [presentity name]);
	}
}

- (void)clearChatView
{
	[_chatDOMView setChat:nil];
	[_chatDOMView loadBaseDocument];
}

- (void)selectLog:(NSManagedObject *)log
{
	if (log) {
		[_loadingProgressIndicator startAnimation:nil];
		
		NSString *path = [log valueForKey:@"path"];
		SavedChat *chat;
		
		if ([[path pathExtension] isEqualToString:@"ichat"]) {
			chat = [[SavedChat alloc] initWithTranscriptFile:path];
		} else {
			NSData *data = [[NSData alloc] initWithContentsOfFile:path];
			chat = [[SavedChat alloc] initWithSavedData:data];
			[data release];
		}
		
		[_chatDOMView scrollToBeginningSmoothly:NO];
		[_chatDOMView setChat:chat];
		[chat release];
		
		[_loadingProgressIndicator stopAnimation:nil];
		[_logTableView setNeedsDisplay:YES];
	} else {
		[self clearChatView];
	}
}

- (void)clearSearchField
{
	if ([[_searchField stringValue] length] > 0) {
		[_searchField setStringValue:@""];
		[self searchTextChanged:_searchField];
	}
}

#pragma mark -
#pragma mark Notification Callbacks
#pragma mark -

- (void)searchTextChanged:(id)sender
{
	//Clear out old search results
	//Begin a search with the given search term, if any
	NSString *searchString = [sender stringValue];
	
	if ([searchString length] > 0) {
		if (_query) {
			[_query stopQuery];
			[_query release];
		}
		
		//Begin a new search with searchString
		_query = [[NSMetadataQuery alloc] init];
		[_query setSearchScopes:[NSArray arrayWithObject:[Prefs savedChatPath]]];
		[_query setPredicate:[NSPredicate predicateWithFormat:@"kMDItemTextContent like[c] %@", searchString]];
		
		if ([_query startQuery]) {
			[[NSNotificationCenter defaultCenter] addObserver:self selector:@selector(spotlightNotificationReceived:) name:NSMetadataQueryDidFinishGatheringNotification object:_query];
		}
	} else {
		//Stop any current search
		if (_query) {
			[_query stopQuery];
			[_query release];
			_query = nil;
			[[NSNotificationCenter defaultCenter] removeObserver:self name:NSMetadataQueryDidFinishGatheringNotification object:_query];
		}
		
		[_logsArrayController setFilterPredicate:nil];
		[_personArrayController setFilterPredicate:nil];
		[_spotlightResults release];
	}
}

- (void)tableViewNotificationReceived:(NSNotification *)note
{
	if ([note object] == _logTableView) {
		[[self window] makeFirstResponder:_personTableView];
	} else {
		[[self window] makeFirstResponder:_logTableView];
		[_logTableView selectRow:0 byExtendingSelection:NO];
	}
}

- (void)spotlightNotificationReceived:(NSNotification *)note
{
	//Extract results out of the query
	int i;
	
	_spotlightResults = [[NSMutableArray alloc] init];
	
	for (i = 0; i < [[note object] resultCount]; i++) {
		NSString *path = [[[note object] resultAtIndex:i] valueForAttribute:@"kMDItemPath"];
		[_spotlightResults addObject:path];
	}
	
	//Do something with the results
	NSFetchRequest *fetchRequest = [[[NSFetchRequest alloc] init] autorelease];
	
	[fetchRequest setEntity:[NSEntityDescription entityForName:@"Log" inManagedObjectContext:[self managedObjectContext]]];
	[fetchRequest setRelationshipKeyPathsForPrefetching:[NSArray arrayWithObject:@"person"]];
	[fetchRequest setPredicate:[NSPredicate predicateWithFormat:@"path IN %@", _spotlightResults]];
	
	NSArray *results = [[self managedObjectContext] executeFetchRequest:fetchRequest error:nil];
	NSMutableArray *people = [NSMutableArray array];
	
	for (NSManagedObject *log in results) {
		[people addObject:[log valueForKey:@"person"]];
	}
	
	[_logsArrayController setFilterPredicate:[NSPredicate predicateWithFormat:@"path IN %@", _spotlightResults]];
	[_personArrayController setFilterPredicate:[NSPredicate predicateWithFormat:@"self IN %@", people]];
	
	[_query stopQuery];
	[[NSNotificationCenter defaultCenter] removeObserver:self name:NSMetadataQueryDidFinishGatheringNotification object:_query];
}

#pragma mark -
#pragma mark Sheet End Methods
#pragma mark -

- (void)resetDatesSheetDidEnd:(NSWindow *)sheet returnCode:(int)returnCode contextInfo:(void  *)contextInfo
{
	if (returnCode == NSOKButton) {
		[self performSelector:@selector(resetLogModificationDates) withObject:nil afterDelay:0.5];
	} else {
		_ignoreDates = YES;
	}
}

- (void)confirmDeleteSheetDidEnd:(NSWindow *)sheet returnCode:(int)returnCode contextInfo:(void  *)contextInfo
{
	if (returnCode == NSOKButton) {
		NSArray *selectedObjects = [_logsArrayController selectedObjects];
		
		if ([[self window] firstResponder] == _logTableView && [selectedObjects count] > 0) {
			//Remove a bunch of logs
			for (NSManagedObject *log in selectedObjects) {
				NSString *source = [log valueForKey:@"path"];
				[[NSWorkspace sharedWorkspace] performFileOperation:NSWorkspaceRecycleOperation source:[source stringByDeletingLastPathComponent] destination:@"" files:[NSArray arrayWithObject:[source lastPathComponent]] tag:nil];
			}
			
			[_logsArrayController removeObjects:selectedObjects];
			
			//Remove the person if no logs are left
			NSManagedObject *person = [[_personArrayController selectedObjects] objectAtIndex:0];
			
			if ([[person valueForKey:@"logs"] count] == 0) {
				[_personArrayController removeObject:person];
			}
		} else {
			//Kill all logs from a single person
			NSArray *selectedObjects = [_personArrayController selectedObjects];
			
			for (NSManagedObject *person in selectedObjects) {
				NSArray *logs = [person valueForKey:@"logs"];
				
				for (NSManagedObject *log in logs) {
					NSString *source = [log valueForKey:@"path"];
					[[NSWorkspace sharedWorkspace] performFileOperation:NSWorkspaceRecycleOperation source:[source stringByDeletingLastPathComponent] destination:@"" files:[NSArray arrayWithObject:[source lastPathComponent]] tag:nil];
				}
				
				[_logsArrayController removeObjects:logs];
				[_personArrayController removeObject:person];
			}
		}
		
		selectedObjects = [_logsArrayController selectedObjects];
		[self selectLog:[selectedObjects count] ? [selectedObjects objectAtIndex:0] : nil];
	}
}

- (void)saveLogSheetDidEnd:(NSSavePanel *)sheet returnCode:(int)returnCode contextInfo:(void *)contextInfo
{
	if (returnCode == NSOKButton) {
		//Get a string representation of the currently selected log
		NSArray *messages = [[_chatDOMView chat] messages];
		NSMutableString *stringRepresentation = [[NSMutableString alloc] init];
		
		for (InstantMessage *message in messages) {
			NSString *time = [[message time] descriptionWithCalendarFormat:@"%H:%M:%S" timeZone:nil locale:nil];
			
			if ([message sender]) {
				[stringRepresentation appendFormat:@"(%@) %@: %@\n", time, [[message sender] name], [[message text] string]];
			} else if ([message subject]) {
				[stringRepresentation appendFormat:@"(%@) %@\n", time, [NSString stringWithFormat:[[message text] string], [[message subject] name]]];
			}
		}
		
		[stringRepresentation writeToFile:[sheet filename] atomically:YES encoding:NSUTF8StringEncoding error:nil];
		[stringRepresentation release];
	}
}

#pragma mark -
#pragma mark Modification Date Correction
#pragma mark -

- (void)resetLogModificationDates
{
	[_resetProgressIndicator setMaxValue:[[[NSFileManager defaultManager] directoryContentsAtPath:[Prefs savedChatPath]] count]];
	[_resetProgressIndicator setDoubleValue:0.0];
	
	[NSApp beginSheet:_resetWindow modalForWindow:[self window] modalDelegate:self didEndSelector:nil contextInfo:nil];
	[NSThread detachNewThreadSelector:@selector(runLogDateReset) toTarget:self withObject:nil];
}

- (void)resetLogIncrementIndicator
{
	[_resetProgressIndicator incrementBy:1.0];
}

- (void)runLogDateReset
{
	NSAutoreleasePool *pool = [[NSAutoreleasePool alloc] init];
	NSEnumerator *enumerator = [[NSFileManager defaultManager] enumeratorAtPath:[Prefs savedChatPath]];
	NSString *nextFile;
	
	while ( (nextFile = [enumerator nextObject]) ) {
		NSString *path = [[Prefs savedChatPath] stringByAppendingPathComponent:nextFile];
		SavedChat *chat = nil;
		
		if ([[path pathExtension] isEqualToString:@"ichat"]) {
			chat = [[SavedChat alloc] initWithTranscriptFile:path];
		} else if ([[path pathExtension] isEqualToString:@"chat"]) {
			NSData *data = [[NSData alloc] initWithContentsOfFile:path];
			chat = [[SavedChat alloc] initWithSavedData:data];
			[data release];
		}
		
		if (chat) {
			[_resetTextField performSelectorOnMainThread:@selector(setStringValue:) withObject:nextFile waitUntilDone:NO];
			[[NSFileManager defaultManager] changeFileAttributes:[NSDictionary dictionaryWithObject:[chat dateCreated] forKey:NSFileCreationDate] atPath:path];
			
			[chat release];
		}
		
		[self performSelectorOnMainThread:@selector(resetLogIncrementIndicator) withObject:nil waitUntilDone:NO];
	}
	
	[NSApp endSheet:_resetWindow];
	[_resetWindow orderOut:nil];
	
	[self reloadLogs];
	
	[pool release];
}

#pragma mark -
#pragma mark Core Data Methods
#pragma mark -

- (NSManagedObjectContext *)managedObjectContext
{
	if (!_managedObjectContext) {
		NSManagedObjectModel *managedObjectModel = [[[NSManagedObjectModel alloc] initWithContentsOfURL:[NSURL fileURLWithPath:[[NSBundle bundleForClass:[Chax class]] pathForResource:@"Logs" ofType:@"mom"]]] autorelease];
		NSPersistentStoreCoordinator *persistentStoreCoordinator = [[[NSPersistentStoreCoordinator alloc] initWithManagedObjectModel:managedObjectModel] autorelease];
		
		[persistentStoreCoordinator addPersistentStoreWithType:NSInMemoryStoreType configuration:nil URL:nil options:nil error:nil];
		
		_managedObjectContext = [[NSManagedObjectContext alloc] init];
		[_managedObjectContext setPersistentStoreCoordinator:persistentStoreCoordinator];
	}
	
	return _managedObjectContext;
}

#pragma mark -
#pragma mark NSToolbar Delegate
#pragma mark -

- (NSArray *)toolbarAllowedItemIdentifiers:(NSToolbar *)toolbar {
	return [NSArray arrayWithObjects:DeleteLogToolbarItemIdentifier, ExportLogToolbarItemIdentifier, RevealLogToolbarItemIdentifier, SearchFieldToolbarItemIdentifier, NSToolbarFlexibleSpaceItemIdentifier, NSToolbarSpaceItemIdentifier, NSToolbarSeparatorItemIdentifier, nil];
}

- (NSArray *)toolbarDefaultItemIdentifiers:(NSToolbar *)toolbar {
	return [NSArray arrayWithObjects:DeleteLogToolbarItemIdentifier, NSToolbarSeparatorItemIdentifier, ExportLogToolbarItemIdentifier, RevealLogToolbarItemIdentifier, NSToolbarFlexibleSpaceItemIdentifier, SearchFieldToolbarItemIdentifier, nil];
}

- (NSToolbarItem *)toolbar:(NSToolbar *)toolbar itemForItemIdentifier:(NSString *)itemIdentifier willBeInsertedIntoToolbar:(BOOL)flag {
	NSToolbarItem *item = [[NSToolbarItem alloc] initWithItemIdentifier:itemIdentifier];
	
	[item setAction:@selector(toolbarItemAction:)];
	[item setTarget:self];
	
	if ([itemIdentifier isEqualToString:DeleteLogToolbarItemIdentifier]) {
		NSImage *deleteImage = [[NSImage alloc] initByReferencingFile:[[NSBundle bundleWithPath:[[NSWorkspace sharedWorkspace] fullPathForApplication:@"Mail.app"]] pathForImageResource:@"delete.tiff"]];
		[item setLabel:ChaxLocalizedString(@"delete")];
		[item setPaletteLabel:ChaxLocalizedString(@"delete")];
		[item setImage:deleteImage];
		[deleteImage release];
	} else if ([itemIdentifier isEqualToString:ExportLogToolbarItemIdentifier]) {
		NSImage *exportImage = [[NSImage alloc] initByReferencingFile:[[NSBundle bundleWithPath:[[NSWorkspace sharedWorkspace] fullPathForApplication:@"TextEdit.app"]] pathForImageResource:@"txt.icns"]];
		[item setLabel:ChaxLocalizedString(@"export_as_text")];
		[item setPaletteLabel:ChaxLocalizedString(@"export_as_text")];
		[item setImage:exportImage];
		[exportImage release];
	} else if ([itemIdentifier isEqualToString:RevealLogToolbarItemIdentifier]) {
		NSImage *finderIcon = [[NSImage alloc] initByReferencingFile:[[NSBundle bundleWithPath:[[NSWorkspace sharedWorkspace] fullPathForApplication:@"Finder.app"]] pathForImageResource:@"Finder.icns"]];
		[item setLabel:ChaxLocalizedString(@"reveal_in_finder")];
		[item setPaletteLabel:ChaxLocalizedString(@"reveal_in_finder")];
		[item setImage:finderIcon];
		[finderIcon release];
	} else if ([itemIdentifier isEqualToString:SearchFieldToolbarItemIdentifier]) {
		[item setLabel:ChaxLocalizedString(@"search")];
		[item setPaletteLabel:ChaxLocalizedString(@"search")];
		
		if (!_searchField) {
			_searchField = [[NSSearchField alloc] initWithFrame:NSMakeRect(0, 0, 22, 50)];
			[_searchField setTarget:self];
			[_searchField setAction:@selector(searchTextChanged:)];
		}
		
		[item setView:_searchField];
		
		[item setMaxSize:NSMakeSize(200, 24)];
		[item setMinSize:NSMakeSize(50, 24)];
		
		[item setImage:nil];
	} else {
		[item release];
		item = nil;
	}
	
	return [item autorelease];
}

- (void)toolbarItemAction:(id)sender {
	if ([[sender itemIdentifier] isEqualToString:DeleteLogToolbarItemIdentifier]) {
		NSString *name = [[[_personArrayController selectedObjects] objectAtIndex:0] valueForKey:@"name"];
		NSArray *selectedLogs = [_logsArrayController selectedObjects];
		
		if (([[self window] firstResponder] == _personTableView && [_personTableView selectedRow] > -1) || [selectedLogs count] > 1) {
			//Prompt to delete a whole bunch of logs
			NSBeginAlertSheet(ChaxLocalizedString(@"delete_logs"), ChaxLocalizedString(@"delete"), ChaxLocalizedString(@"dont_delete"), nil, [self window], self, NULL, @selector(confirmDeleteSheetDidEnd:returnCode:contextInfo:) , nil, [NSString stringWithFormat:ChaxLocalizedString(([[self window] firstResponder] == _personTableView && [_personTableView selectedRow] > -1) ? @"delete_all_logs_msg" : @"delete_selected_logs_msg"), name]);
		} else {
			NSManagedObject *log = [selectedLogs objectAtIndex:0], *person = [log valueForKey:@"person"];
			NSString *source = [log valueForKey:@"path"];
			
			[[NSWorkspace sharedWorkspace] performFileOperation:NSWorkspaceRecycleOperation source:[source stringByDeletingLastPathComponent] destination:@"" files:[NSArray arrayWithObject:[source lastPathComponent]] tag:nil];
			
			if ([_logTableView selectedRow] == [[_logsArrayController arrangedObjects] count] - 1) {
				//The last row is selected, so select one row above
				[_logTableView selectRow:[_logTableView selectedRow] - 1 byExtendingSelection:NO];
			}
			
			//Remove the log from the Core Data store
			[_logsArrayController removeObject:[selectedLogs objectAtIndex:0]];
			
			//Remove the person if no logs are left
			if ([[person valueForKey:@"logs"] count] == 0) {
				[_personArrayController removeObject:person];
			}
			
			//See if a new log was selected
			selectedLogs = [_logsArrayController selectedObjects];
			[self selectLog:[selectedLogs count] ? [selectedLogs objectAtIndex:0] : nil];
		}
	} else if ([[sender itemIdentifier] isEqualToString:ExportLogToolbarItemIdentifier]) {
		NSSavePanel *panel = [NSSavePanel savePanel];
		[panel setCanSelectHiddenExtension:YES];
		[panel setCanCreateDirectories:YES];
		[panel setRequiredFileType:@"txt"];
		[panel beginSheetForDirectory:nil file:[[_chatDOMView chat] defaultSaveName] modalForWindow:[self window] modalDelegate:self didEndSelector:@selector(saveLogSheetDidEnd:returnCode:contextInfo:) contextInfo:nil];
	} else if ([[sender itemIdentifier] isEqualToString:RevealLogToolbarItemIdentifier]) {
		if ([_logTableView selectedRow] > -1) {
			NSString *path = [[[_logsArrayController selectedObjects] objectAtIndex:0] valueForKey:@"path"];
			[[NSWorkspace sharedWorkspace] selectFile:path inFileViewerRootedAtPath:[path stringByDeletingLastPathComponent]];
		}
	} else if ([[sender itemIdentifier] isEqualToString:SearchFieldToolbarItemIdentifier]) {
		//Do nothing.
	}
}

- (void)toolbarDidRemoveItem:(NSNotification *)notification
{
	NSEnumerator *itemEnumerator = [[[notification object] items] objectEnumerator];
	NSToolbarItem *nextItem;
	
	while ( (nextItem = [itemEnumerator nextObject]) ) {
		if ([[nextItem itemIdentifier] isEqualToString:SearchFieldToolbarItemIdentifier]) {
			return;
		}
	}
	
	[self clearSearchField];
}

- (BOOL)validateToolbarItem:(NSToolbarItem *)item
{
	if ([[item itemIdentifier] isEqualToString:DeleteLogToolbarItemIdentifier]) {
		return [_personTableView selectedRow] > -1;
	} else if ([[item itemIdentifier] isEqualToString:ExportLogToolbarItemIdentifier] || [[item itemIdentifier] isEqualToString:RevealLogToolbarItemIdentifier]) {
		return [_logTableView selectedRow] > -1;
	}
	return YES;
}

#pragma mark -
#pragma mark NSTableView Methods
#pragma mark -

- (BOOL)tableView:(NSTableView *)tableView shouldSelectRow:(int)rowIndex
{
	return _firstLoadComplete;
}

- (void)tableViewSelectionDidChange:(NSNotification *)note
{
	[self performSelectorOnMainThread:@selector(tableViewSelectionChanged:) withObject:[note object] waitUntilDone:NO];
}

- (void)tableViewSelectionChanged:(NSTableView *)tableView
{
	NSArray *selectedLogs = [_logsArrayController selectedObjects];
	
	[self selectLog:[selectedLogs count] ? [selectedLogs objectAtIndex:0] : nil];
}

@end
