/*
 * Prefs_Chax.m
 *
 * Copyright (c) 2007-2009 Kent Sutherland
 * 
 * Permission is hereby granted, free of charge, to any person obtaining a copy of
 * this software and associated documentation files (the "Software"), to deal in
 * the Software without restriction, including without limitation the rights to use,
 * copy, modify, merge, publish, distribute, sublicense, and/or sell copies of the
 * Software, and to permit persons to whom the Software is furnished to do so,
 * subject to the following conditions:
 * 
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 * 
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS
 * FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR
 * COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER
 * IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
 * CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 */

#import "Prefs_Chax.h"
#import "iChat4.h"
#import "ChaxPrefsView.h"
#import "AutoAcceptPrefsWindowController.h"
#import "Chax.h"
#import "UpdateController.h"
#import "ChaxDefaults.h"
#import "Chax_FezPreferences.h"

#define SENDER_STATE ([(NSCell *)sender state] == NSOnState)
#define PREF_STATE(x) [Chax boolForKey:x] ? NSOnState : NSOffState

#define CONTACT_FONT_TAG					101
#define STATUS_TEXT_FONT_TAG				102

@implementation Prefs_Chax

- (void)awakeFromNib
{
	//Set the version in the about tab
	[_versionTextField setStringValue:[NSString stringWithFormat:@"Chax %@", [[NSBundle bundleForClass:[self class]] objectForInfoDictionaryKey:@"CFBundleVersion"]]];
	
	//Restore font settings
	NSData *data = [Chax dataForKey:@"ContactListFont"];
	NSFont *font = (data != nil) ? [NSUnarchiver unarchiveObjectWithData:data] : nil;
	_changingFont = CONTACT_FONT_TAG;
	[self setContactFont:font ? font : [NSFont fontWithName:@"LucidaGrande" size:12]];
	
	data = [Chax dataForKey:@"ContactListStatusFont"];
	font = (data != nil) ? [NSUnarchiver unarchiveObjectWithData:data] : nil;
	_changingFont = STATUS_TEXT_FONT_TAG;
	[self setContactFont:font ? font : [NSFont fontWithName:@"LucidaGrande" size:11]];
}

- (id)init
{
	if ( (self = [super init]) ) {
		_defaults = [[ChaxDefaults alloc] init];
		
		[NSBundle loadNibNamed:[self preferencesNibName] owner:self];
		[(ChaxPrefsView *)_view setController:self];
		
		_icon = [[NSImage alloc] initWithContentsOfFile:[[NSBundle bundleWithIdentifier:@"com.ksuther.chax"] pathForImageResource:@"Chax"]];
	}
	return self;
}

- (void)dealloc
{
	[_icon release];
	[_defaults release];
	[super dealloc];
}

#pragma mark -
#pragma mark NSPreferenceModule Methods
#pragma mark -

- (NSView *)viewForPreferenceNamed:(NSString *)name
{
	return _view;
}

- (NSImage *)imageForPreferenceNamed:(NSString *)name
{
	return _icon;
}

- (NSString *)preferencesNibName
{
    return @"ChaxPreferences";
}

- (BOOL)isResizable
{
    return NO;
}

- (NSSize)minSize
{
	return [_view frame].size;
}

#pragma mark -
#pragma mark IBActions
#pragma mark -

- (IBAction)aboutAction:(id)sender
{
	switch ([sender tag]) {
		case 500:
			//Check for updates
			[[UpdateController sharedController] checkForUpdates:nil];
			break;
		case 502:
			//Donate
			[[NSWorkspace sharedWorkspace] openURL:DONATE_URL];
			break;
	}
}

- (IBAction)changeContactListSetting:(id)sender
{
	switch ([sender tag]) {
		case CONTACT_FONT_TAG:
			//Set font for contacts
			_changingFont = CONTACT_FONT_TAG;
			[[NSFontManager sharedFontManager] setSelectedFont:_contactFont isMultiple:NO];
			[[NSFontManager sharedFontManager] orderFrontFontPanel:self];
			break;
		case STATUS_TEXT_FONT_TAG:
			//Set font for status text
			_changingFont = STATUS_TEXT_FONT_TAG;
			[[NSFontManager sharedFontManager] setSelectedFont:_statusMessageFont isMultiple:NO];
			[[NSFontManager sharedFontManager] orderFrontFontPanel:self];
			break;
	}
}

- (IBAction)showAutoAcceptOptions:(id)sender
{
	//Load auto-accept settings depending on which button was clicked
	[_autoAcceptPrefs setAutoAcceptType:[sender tag]];
	[NSApp beginSheet:[_autoAcceptPrefs window] modalForWindow:[[FezPreferences sharedPreferences] chax_preferencesPanel] modalDelegate:nil didEndSelector:nil contextInfo:nil];
}

#pragma mark -
#pragma mark Accessors
#pragma mark -

- (void)setAutoresizeContactList:(BOOL)value
{
	[_defaults setValue:[NSNumber numberWithBool:value] forKey:@"AutoresizeContactList"];
	
	if (value) {
		[[NSNotificationCenter defaultCenter] postNotificationName:@"ReloadContactList" object:nil];
	}
}

- (BOOL)autoresizeContactList
{
	return [[_defaults valueForKey:@"AutoresizeContactList"] boolValue];
}

- (void)setUseCustomContactListFonts:(BOOL)value
{
	[[NSNotificationCenter defaultCenter] postNotificationName:@"ReloadContactList" object:nil];
	
	[_defaults setValue:[NSNumber numberWithBool:value] forKey:@"UseCustomContactListFonts"];
}

- (BOOL)useCustomContactListFonts
{
	return [[_defaults valueForKey:@"UseCustomContactListFonts"] boolValue];
}

- (void)setAutoAcceptTextInvitations:(BOOL)value
{
	[_defaults setValue:[NSNumber numberWithBool:value] forKey:@"SkipNewMessageNotification"];
	[Prefs setKnockKnock:!value];
}

- (BOOL)autoAcceptTextInvitations
{
	return [[_defaults valueForKey:@"SkipNewMessageNotification"] boolValue];
}

- (void)setGrowlEnabled:(BOOL)value
{
	if (value) {
		NSRunInformationalAlertPanel(ChaxLocalizedString(@"growl_prefs_title"), ChaxLocalizedString(@"growl_prefs_msg"), nil, nil, nil);
	}
	
	[_defaults setValue:[NSNumber numberWithBool:value] forKey:@"GrowlEnabled"];
}

- (BOOL)growlEnabled
{
	return [[_defaults valueForKey:@"GrowlEnabled"] boolValue];
}

- (void)setShowMenuBadge:(BOOL)value
{
	NSImage *badgeImage = nil;
	if (value) {
		badgeImage = [[[NSImage alloc] initByReferencingFile:[[NSBundle bundleWithIdentifier:@"com.ksuther.chax"] pathForImageResource:@"menu-badge"]] autorelease];
		[Chax setBool:YES forKey:@"ShowMenuBadge"];
	} else {
		[Chax setBool:NO forKey:@"ShowMenuBadge"];
	}
	[[Chax menuItems] makeObjectsPerformSelector:@selector(setImage:) withObject:badgeImage];
	
	[_defaults setValue:[NSNumber numberWithBool:value] forKey:@"ShowMenuBadge"];
}

- (BOOL)showMenuBadge
{
	return [[_defaults valueForKey:@"ShowMenuBadge"] boolValue];
}

- (void)setAutoAcceptFiles:(BOOL)value
{
	if (value) {
		NSBeginAlertSheet(ChaxLocalizedString(@"auto_accept_files_title"), ChaxLocalizedString(@"no"), ChaxLocalizedString(@"yes"), nil, [[FezPreferences sharedPreferences] chax_preferencesPanel], self, @selector(confirmSheetDidEnd:returnCode:contextInfo:), nil, (void *)1, ChaxLocalizedString(@"auto_accept_files_msg"));
	} else {
		[_defaults setValue:[NSNumber numberWithBool:value] forKey:@"AutoAcceptFiles"];
	}
}

- (BOOL)AutoAcceptFiles
{
	return [[_defaults valueForKey:@"AutoAcceptFiles"] boolValue];
}

- (void)setAutoAcceptAVChats:(BOOL)value
{
	if (value) {
		NSBeginAlertSheet(ChaxLocalizedString(@"auto_accept_av_title"), ChaxLocalizedString(@"no"), ChaxLocalizedString(@"yes"), nil, [[FezPreferences sharedPreferences] chax_preferencesPanel], self, @selector(confirmSheetDidEnd:returnCode:contextInfo:), nil, (void *)2, ChaxLocalizedString(@"auto_accept_av_msg"));
	} else {
		[_defaults setValue:[NSNumber numberWithBool:value] forKey:@"AutoAcceptAVChats"];
	}
}

- (BOOL)AutoAcceptAVChats
{
	return [[_defaults valueForKey:@"AutoAcceptAVChats"] boolValue];
}

- (void)setAutoAcceptScreenSharing:(BOOL)value
{
	if (value) {
		NSBeginAlertSheet(ChaxLocalizedString(@"auto_accept_sharing_title"), ChaxLocalizedString(@"no"), ChaxLocalizedString(@"yes"), nil, [[FezPreferences sharedPreferences] chax_preferencesPanel], self, @selector(confirmSheetDidEnd:returnCode:contextInfo:), nil, (void *)3, ChaxLocalizedString(@"auto_accept_sharing_msg"));
	} else {
		[_defaults setValue:[NSNumber numberWithBool:value] forKey:@"AutoAcceptScreenSharing"];
	}
}

- (BOOL)AutoAcceptScreenSharing
{
	return [[_defaults valueForKey:@"AutoAcceptScreenSharing"] boolValue];
}

- (void)setHideContactLists:(BOOL)value
{
	[_defaults setValue:[NSNumber numberWithBool:value] forKey:@"HideContactListsWhenInactive"];
	
	NSArray *controllers = [PeopleListController peopleListControllers];
	
	for (PeopleListController *c in controllers) {
		[[c window] setHidesOnDeactivate:value];
		
		if ([c isVisible]) {
			[c displayWithoutKey];
		}
	}
}

- (BOOL)hideContactLists
{
	return [[_defaults valueForKey:@"HideContactListsWhenInactive"] boolValue];
}

- (void)setHideChatWindows:(BOOL)value
{
	[_defaults setValue:[NSNumber numberWithBool:value] forKey:@"HideChatsWhenInactive"];
	
	NSArray *controllers = [ChatWindowController allChatWindowControllers];
	
	for (ChatWindowController *c in controllers) {
		if ([[c currentChatController] notifier]) {
			[[[[c currentChatController] notifier] floaterWindow] setHidesOnDeactivate:value];
			[[[[c currentChatController] notifier] floaterWindow] orderFront:nil];
		} else {
			[[c window] setHidesOnDeactivate:value];
			[[c window] orderBack:nil];
		}
	}
}

- (BOOL)hideChatWindows
{
	return [[_defaults valueForKey:@"HideChatsWhenInactive"] boolValue];
}

- (void)setContactFont:(NSFont *)font
{
	NSTextField *field = nil;
	NSString *key = nil;
	
	switch (_changingFont) {
		case CONTACT_FONT_TAG:
			[_contactFont release];
			_contactFont = [font retain];
			field = _contactFontTextField;
			key = @"ContactListFont";
			break;
		case STATUS_TEXT_FONT_TAG:
			[_statusMessageFont release];
			_statusMessageFont = [font retain];
			field = _statusMessageFontTextField;
			key = @"ContactListStatusFont";
			break;
	}
	
	if (field && key) {
		[field setStringValue:[NSString stringWithFormat:@"%@ %.0f", [font displayName], [font pointSize]]];
		[Chax setObject:[NSArchiver archivedDataWithRootObject:font] forKey:key];
		[[NSNotificationCenter defaultCenter] postNotificationName:@"ReloadContactList" object:nil];
	}
}

- (NSFont *)contactFont
{
	return _contactFont;
}

- (void)setShowMessageHistory:(BOOL)value
{
	if (value && ![Prefs autosaveChats]) {
		NSString *title = ChaxLocalizedString(@"message_history_logs_title");
		NSString *message = ChaxLocalizedString(@"message_history_logs_msg");
		NSString *enableButton = ChaxLocalizedString(@"message_history_enable_logging");
		NSString *cancelButton = ChaxLocalizedString(@"cancel");
		
		NSBeginAlertSheet(title, 
						  enableButton,
						  cancelButton,
						  nil,
						  [[FezPreferences sharedPreferences] chax_preferencesPanel],
						  self,
						  @selector(messageHistorySheetDidEnd:returnCode:contextInfo:),
						  NULL,
						  NULL,
						  message);
	} else {
		[_defaults setValue:[NSNumber numberWithBool:value] forKey:@"ShowMessageHistory"];
	}
}

- (BOOL)showMessageHistory
{
	return [[_defaults valueForKey:@"ShowMessageHistory"] boolValue];
}

#pragma mark -
#pragma mark Sheet Callback
#pragma mark -

- (void)messageHistorySheetDidEnd:(NSWindow *)sheet returnCode:(NSInteger)returnCode contextInfo:(void *)contextInfo
{
	[self willChangeValueForKey:@"showMessageHistory"];
	
	if (returnCode == NSOKButton) {
		[Prefs setAutosaveChats:YES];
		[_defaults setValue:[NSNumber numberWithBool:YES] forKey:@"ShowMessageHistory"];
	}
	
	[self didChangeValueForKey:@"showMessageHistory"];
}

- (void)confirmSheetDidEnd:(NSWindow *)sheet returnCode:(int)returnCode contextInfo:(void *)contextInfo
{
	BOOL value = (returnCode != NSOKButton);
	
	switch ((int)contextInfo) {
		case 1:
			[self willChangeValueForKey:@"AutoAcceptFiles"];
			[_defaults setValue:[NSNumber numberWithBool:value] forKey:@"AutoAcceptFiles"];
			[self didChangeValueForKey:@"AutoAcceptFiles"];
			break;
		case 2:
			[self willChangeValueForKey:@"AutoAcceptAVChats"];
			[_defaults setValue:[NSNumber numberWithBool:value] forKey:@"AutoAcceptAVChats"];
			[self didChangeValueForKey:@"AutoAcceptAVChats"];
			break;
		case 3:
			[self willChangeValueForKey:@"AutoAcceptScreenSharing"];
			[_defaults setValue:[NSNumber numberWithBool:value] forKey:@"AutoAcceptScreenSharing"];
			[self didChangeValueForKey:@"AutoAcceptScreenSharing"];
			break;
	}
}

@end
